/*
	File:		NewtonTypes.h

	Contains:	Global types for Newton build system.

	Written by:	Newton Research Group.
*/

#if !defined(__NEWTONTYPES_H)
#define __NEWTONTYPES_H 1


/* Base types */

typedef signed char		SChar;
typedef unsigned char 	UChar;

typedef signed char		SByte;
typedef unsigned char	UByte;

typedef signed short		SShort;
typedef unsigned short	UShort;

typedef signed long		SLong;
typedef unsigned long	ULong;

typedef long				NewtonErr;	/* needs to hold error codes less than -32767 */

/* The type of objects in the OS */

typedef unsigned long	ObjectId;

/* Address types */

typedef unsigned long	VAddr;
typedef unsigned long	PAddr;

/* Ref types */

typedef long Ref;

typedef struct
{
	Ref	ref;
	Ref	stackPos;
} RefHandle;

#if defined(__cplusplus)
class RefVar;
typedef const RefVar & RefArg;
#else
typedef RefHandle * RefVar;
typedef RefVar RefStruct;
typedef const RefVar RefArg;
#endif

typedef Ref (*MapSlotsFunction)(RefArg tag, RefArg value, unsigned anything);


#if !defined(nil)
#define nil 0
#endif

typedef signed char BOOL;
#define YES (BOOL)1
#define NO  (BOOL)0

#if !defined(__MACTYPES__)
#define __MACTYPES__ 1
#if __LP64__
typedef unsigned int                    UInt32;
typedef signed int                      SInt32;
#else
typedef unsigned long                   UInt32;
typedef signed long                     SInt32;
#endif

/* avoid redeclaration if libkern/OSTypes.h */
#ifndef _OS_OSTYPES_H
#if TARGET_RT_BIG_ENDIAN
struct wide {
  SInt32              hi;
  UInt32              lo;
};
typedef struct wide                     wide;
struct UnsignedWide {
  UInt32              hi;
  UInt32              lo;
};
typedef struct UnsignedWide             UnsignedWide;
#else
struct wide {
  UInt32              lo;
  SInt32              hi;
};
typedef struct wide                     wide;
struct UnsignedWide {
  UInt32              lo;
  UInt32              hi;
};
typedef struct UnsignedWide             UnsignedWide;
#endif  /* TARGET_RT_BIG_ENDIAN */
#endif

typedef unsigned short	UniChar;


/* Pointer types */

typedef char *		Ptr;
typedef Ptr *		Handle;
typedef long		(*ProcPtr)(void*);


/* Math types 

typedef struct
{
	SLong	hi;
	ULong	lo;
} wide;
*/
typedef long Fixed;
typedef long Fract;
typedef unsigned long UnsignedFixed;


/* Graphics types */

typedef struct
{
	short	v;
	short	h;
} Point;


typedef struct
{
	short	top;
	short	left;
	short	bottom;
	short	right;
} Rect;


typedef struct
{
	Fixed	x;
	Fixed	y;
} FixedPoint;


typedef struct
{
	Fixed	left;
	Fixed	top;
	Fixed	right;
	Fixed	bottom;
} FixedRect;

#endif	/* __MACTYPES__ */


#if defined(__cplusplus)

class SingleObject {};

inline ProcPtr
ptmf2ptf(long (SingleObject::*func)(int,void*))
{
	union
	{
		long (SingleObject::*fIn)(int,void*);
		ProcPtr	fOut;
	} map;

	map.fIn = func;
	return map.fOut;
}
#define MemberFunctionCast(_t, self, fn) (_t) ptmf2ptf((long (SingleObject::*)(int,void*)) fn)

#endif	/* __cplusplus */

#endif	/* __NEWTONTYPES_H */
